package com.example.meetup;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.Query;
import com.google.firebase.firestore.WriteBatch;

import android.widget.TextView;
import java.time.LocalTime;
import java.util.ArrayList;

import java.util.HashMap;
import java.util.List;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.Map;


public class AdminPanelActivity extends AppCompatActivity {

    private FloatingActionButton addEventButton;
    private RecyclerView eventsRecyclerView;
    private List<Event> eventsList = new ArrayList<>();
    private EventsAdapter eventsAdapter;
    private FirebaseFirestore db = FirebaseFirestore.getInstance();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_admin_panel);

        eventsAdapter = new EventsAdapter(eventsList, this::showEventActionsDialog);
        addEventButton = findViewById(R.id.addEventButton);
        eventsRecyclerView = findViewById(R.id.eventsRecyclerView);

        eventsRecyclerView.setLayoutManager(new LinearLayoutManager(this));
        eventsRecyclerView.setAdapter(eventsAdapter);
        addEventButton.setOnClickListener(view -> showAddEventDialog());

        // Загрузка событий из Firestore
        loadEvents();
    }

    private void loadEvents() {
        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        if (currentUser != null) {
            String userId = currentUser.getUid();
            db.collection("users").document(userId).collection("events")
                    .orderBy("date", Query.Direction.DESCENDING)
                    .get()
                    .addOnSuccessListener(queryDocumentSnapshots -> {
                        eventsList.clear(); // Очищаем список перед добавлением новых данных
                        for (DocumentSnapshot documentSnapshot : queryDocumentSnapshots.getDocuments()) {
                            Event event = documentSnapshot.toObject(Event.class);
                            if (event != null) {
                                event.setId(documentSnapshot.getId());
                                eventsList.add(event);
                            }
                        }
                        eventsAdapter.notifyDataSetChanged(); // Обновляем адаптер
                    })

                    .addOnFailureListener(e -> Toast.makeText(AdminPanelActivity.this, "Error loading events", Toast.LENGTH_SHORT).show());
        }
    }

    private void showEventActionsDialog(Event event) {
        CharSequence[] actions = {"Добавить приглашение", "Показать приглашения","Показать код", "Удалить"};

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setItems(actions, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                switch (which) {
                    case 0: // Add Invitation
                        showAddInvitationDialog(event);
                        break;
                    case 1: // View Invitations
                        viewInvitations(event);
                        break;
                    case 2: // Code Event
                        showEventCode(event);
                        break;
                    case 3: // Delete Event
                        deleteEvent(event);
                        break;
                }
            }
        });
        builder.show();
    }
    private void showEventCode(Event event) {
        // Создаем диалоговое окно для отображения кода пользователя и события
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Код пользователя и мероприятия");

        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        String userId = currentUser != null ? currentUser.getUid() : "Нет данных о пользователе";
        String eventId = event.getId();

        // Создаем LinearLayout для вертикального размещения TextView
        LinearLayout layout = new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);
        layout.setPadding(50, 20, 50, 20); // Небольшой отступ для улучшения визуализации

        // Создаем TextView для отображения userId
        TextView userView = new TextView(this);
        userView.setText("UserID: " + userId);
        userView.setTextIsSelectable(true); // Позволяет пользователю выделить и скопировать текст

        // Создаем TextView для отображения eventId
        TextView eventView = new TextView(this);
        eventView.setText("EventID: " + eventId);
        eventView.setTextIsSelectable(true); // Позволяет пользователю выделить и скопировать текст

        // Добавляем TextView в LinearLayout
        layout.addView(userView);
        layout.addView(eventView);

        // Устанавливаем LinearLayout как вид для диалогового окна
        builder.setView(layout);

        builder.setPositiveButton("OK", (dialog, which) -> dialog.dismiss());

        builder.show();
    }

    private void showAddEventDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Добавить мероприятие");

        // Set up the input
        final EditText inputName = new EditText(this);
        inputName.setHint("Название");
        final EditText inputDate = new EditText(this);
        inputDate.setHint("Дата");
        final EditText inputTime = new EditText(this);
        inputTime.setHint("Время");
        final EditText inputPlace = new EditText(this);
        inputPlace.setHint("Место");

        LinearLayout layout = new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);
        layout.addView(inputName);
        layout.addView(inputDate);
        layout.addView(inputTime);
        layout.addView(inputPlace);
        inputDate.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                // Проверка корректности введенной даты
                try {
                    LocalDate eventDate = LocalDate.parse(s.toString(), DateTimeFormatter.ISO_LOCAL_DATE);
                    LocalDate now = LocalDate.now();
                    if (eventDate.isBefore(now)) {
                        inputDate.setError("Дата не может быть в прошлом");
                    }
                } catch (DateTimeParseException e) {
                    inputDate.setError("Неправильный формат даты, ожидается ГГГГ-ММ-ДД");
                }
            }
        });
        inputTime.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                // Не требуется изменений
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                // Не требуется изменений
            }

            @Override
            public void afterTextChanged(Editable s) {
                try {
                    LocalTime eventTime = LocalTime.parse(s.toString(), DateTimeFormatter.ISO_LOCAL_TIME);
                    LocalTime now = LocalTime.now();
                    // Так как мы проверяем только время, то важно, что бы время было позже текущего, если дата мероприятия - сегодняшняя
                    LocalDate today = LocalDate.now();
                    LocalDate eventDate = LocalDate.parse(inputDate.getText().toString(), DateTimeFormatter.ISO_LOCAL_DATE); // Убедитесь, что inputDate содержит корректную дату
                    if (!eventDate.isAfter(today) && !eventTime.isAfter(now)) { // Проверяем, что если событие сегодня, время должно быть позже текущего
                        inputTime.setError("Время не может быть в прошлом или сейчас");
                    } else {
                        inputTime.setError(null); // Очистка ошибки, если время корректно
                    }
                } catch (DateTimeParseException e) {
                    inputTime.setError("Неправильный формат времени, ожидается ЧЧ:мм");
                }
            }
        });


        builder.setView(layout);

        // Set up the buttons
        builder.setPositiveButton("Добавить", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                String eventName = inputName.getText().toString();
                String eventDate = inputDate.getText().toString();
                String eventTime = inputTime.getText().toString();
                String eventPlace = inputPlace.getText().toString();
                addEvent(eventName, eventDate, eventTime, eventPlace);
            }
        });
        builder.setNegativeButton("Отмена", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                dialog.cancel();
            }
        });

        builder.show();
    }

    private void addEvent(String name, String date, String time, String place) {
        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        if (currentUser != null) {
            String userId = currentUser.getUid();
            Map<String, Object> event = new HashMap<>();
            event.put("name", name);
            event.put("date", date);
            event.put("time", time);
            event.put("place", place);
            // Добавление мероприятия в Firestore
            FirebaseFirestore db = FirebaseFirestore.getInstance();
            db.collection("users").document(userId).collection("events")
                    .add(event)
                    .addOnSuccessListener(documentReference -> {
                        Toast.makeText(AdminPanelActivity.this, "Мероприятие добавлено", Toast.LENGTH_SHORT).show();
                        loadEvents(); // Загружаем события заново, чтобы обновить список
                    })
                    .addOnFailureListener(e -> Toast.makeText(AdminPanelActivity.this, "Ошибка при добавлении", Toast.LENGTH_SHORT).show());

        } else {
            // Пользователь не аутентифицирован
            Toast.makeText(this, "Вы должны авторизоваться", Toast.LENGTH_SHORT).show();
        }
    }




    private void viewInvitations(Event event) {
        Intent intent = new Intent(this, InvitationsActivity.class);
        intent.putExtra("eventId", event.getId());
        startActivity(intent);
    }
    private void showAddInvitationDialog(Event event) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);

        final EditText input = new EditText(this);
        input.setInputType(InputType.TYPE_CLASS_TEXT);
        input.setHint("ФИО");
        builder.setView(input);

        builder.setPositiveButton("Добавить", (dialog, which) -> {
            String fullName = input.getText().toString().trim();
            if (!fullName.isEmpty()) {
                addInvitationToFirestore(event.getId(), fullName);
            } else {
                Toast.makeText(AdminPanelActivity.this, "Неверный ввод", Toast.LENGTH_SHORT).show();
            }
        });

        builder.setNegativeButton("Отмена", (dialog, which) -> dialog.cancel());

        builder.show();
    }

    private void addInvitationToFirestore(String eventId, String fullName) {
        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        if (currentUser != null && eventId != null) {
            String userId = currentUser.getUid();
            Map<String, Object> invitation = new HashMap<>();
            invitation.put("fullName", fullName);
            invitation.put("isAccepted", false);
            invitation.put("qrCodeString","");
            FirebaseFirestore db = FirebaseFirestore.getInstance();
            db.collection("users").document(userId)
                    .collection("events").document(eventId)
                    .collection("invitations")
                    .add(invitation)
                    .addOnSuccessListener(documentReference -> {
                        Toast.makeText(AdminPanelActivity.this, "Приглашение добавлено", Toast.LENGTH_SHORT).show();;
                    })
                    .addOnFailureListener(e -> {
                        Toast.makeText(AdminPanelActivity.this, "Ошибка при добавлении", Toast.LENGTH_SHORT).show();
                    });
        } else {
            Toast.makeText(this, "Вы должны авторизоваться", Toast.LENGTH_SHORT).show();
        }
    }

    private void deleteEvent(Event event) {
        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        if (currentUser != null) {
            String userId = currentUser.getUid();
            CollectionReference invitationsRef = db.collection("users").document(userId)
                    .collection("events").document(event.getId())
                    .collection("invitations");

            // Удаление всех приглашений перед удалением события
            invitationsRef.get().addOnCompleteListener(task -> {
                if (task.isSuccessful()) {
                    // Создание пакета для удаления документов
                    WriteBatch batch = db.batch();
                    for (DocumentSnapshot invitationSnapshot : task.getResult()) {
                        batch.delete(invitationSnapshot.getReference());
                    }

                    // Применение пакета
                    batch.commit().addOnCompleteListener(batchTask -> {
                        if (batchTask.isSuccessful()) {
                            // Теперь можно удалять само событие
                            db.collection("users").document(userId).collection("events").document(event.getId())
                                    .delete()
                                    .addOnSuccessListener(aVoid -> {
                                        Toast.makeText(AdminPanelActivity.this, "Мероприятие и все приглашения удалены", Toast.LENGTH_SHORT).show();
                                        // Обновление UI после успешного удаления
                                        loadEvents();
                                    })
                                    .addOnFailureListener(e -> Toast.makeText(AdminPanelActivity.this, "Ошибка при удалении мероприятия", Toast.LENGTH_SHORT).show());
                        } else {
                            Toast.makeText(AdminPanelActivity.this, "Ошибка при удалении приглашений", Toast.LENGTH_SHORT).show();
                        }
                    });
                } else {
                    Toast.makeText(AdminPanelActivity.this, "Ошибка при получении приглашений", Toast.LENGTH_SHORT).show();
                }
            });
        } else {
            Toast.makeText(this, "Вы должны авторизоваться", Toast.LENGTH_SHORT).show();
        }
    }
}

