package com.example.meetup;

import android.graphics.Bitmap;
import android.util.Base64;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;

import java.io.ByteArrayOutputStream;

public class Invitation {
    private String id;
    private String fullName;
    private String qrCodeString; // Строка QR-кода
    private boolean isAccepted;
    // Конструктор по умолчанию необходим для Firebase
    public Invitation() {
    }

    // Конструктор с параметрами
    public Invitation(String fullName, String qrCodeString) {
        this.fullName = fullName;
        this.qrCodeString = qrCodeString;
    }

    // Геттеры и сеттеры
    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getFullName() {
        return fullName;
    }

    public String getQrCodeString() {
        return qrCodeString;
    }

    public void setQrCodeString(String qrCodeString) {
        this.qrCodeString = qrCodeString;
    }

    // Метод для генерации QR-кода, замените его реализацией в соответствии с вашими требованиями
    // Этот метод примерный и должен быть реализован в соответствии с логикой вашего приложения
    public static String generateQRCodeString(String userId, String eventId, String invitationId) {
        String data = userId + ":" + eventId + ":" + invitationId;
        try {
            Bitmap bitmap = encodeAsBitmap(data, BarcodeFormat.QR_CODE, 1000, 1000);
            return bitmapToBase64(bitmap);
        } catch (WriterException e) {
            e.printStackTrace();
            return null;
        }
    }

    private static Bitmap encodeAsBitmap(String data, BarcodeFormat format, int width, int height) throws WriterException {
        BitMatrix result;
        try {
            result = new MultiFormatWriter().encode(data, format, width, height, null);
        } catch (IllegalArgumentException iae) {
            // Unsupported format
            return null;
        }
        int w = result.getWidth();
        int h = result.getHeight();
        int[] pixels = new int[w * h];
        for (int y = 0; y < h; y++) {
            int offset = y * w;
            for (int x = 0; x < w; x++) {
                pixels[offset + x] = result.get(x, y) ? 0xFF000000 : 0xFFFFFFFF;
            }
        }
        Bitmap bitmap = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
        bitmap.setPixels(pixels, 0, width, 0, 0, w, h);
        return bitmap;
    }

    private static String bitmapToBase64(Bitmap bitmap) {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        bitmap.compress(Bitmap.CompressFormat.PNG, 100, byteArrayOutputStream);
        byte[] byteArray = byteArrayOutputStream.toByteArray();
        return Base64.encodeToString(byteArray, Base64.DEFAULT);
    }
}


