package com.example.meetup;

import android.app.Dialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.ColorDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.provider.MediaStore;
import android.text.InputType;
import android.util.Base64;
import android.util.Log;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.Toast;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.QueryDocumentSnapshot;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class InvitationsActivity extends AppCompatActivity implements InvitationsAdapter.OnInvitationClickListener {
    private RecyclerView recyclerView;
    private InvitationsAdapter adapter;
    private List<Invitation> invitations = new ArrayList<>();
    private FirebaseFirestore db = FirebaseFirestore.getInstance();
    private String eventId;
    private FloatingActionButton addInvitationButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_invitations);

        recyclerView = findViewById(R.id.recyclerViewInvitations);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));
        adapter = new InvitationsAdapter(invitations, this); // Инициализируем адаптер с пустым списком
        recyclerView.setAdapter(adapter);

        eventId = getIntent().getStringExtra("eventId");
        if (eventId != null) {
            loadInvitations(eventId);
        }

        addInvitationButton = findViewById(R.id.addInvitationButton);
        addInvitationButton.setOnClickListener(view -> {
            // Открытие диалога для добавления приглашения
            showAddInvitationDialog();
        });
    }
    private void showAddInvitationDialog() {
        // Реализуйте метод для отображения диалогового окна добавления приглашения
        AlertDialog.Builder builder = new AlertDialog.Builder(this);

        final EditText input = new EditText(this);
        input.setInputType(InputType.TYPE_CLASS_TEXT);
        input.setHint("ФИО");
        builder.setView(input);

        builder.setPositiveButton("Добавить", (dialog, which) -> {
            String fullName = input.getText().toString().trim();
            if (!fullName.isEmpty()) {
                addInvitationToFirestore(eventId, fullName);
            } else {
                Toast.makeText(InvitationsActivity.this, "Неправильный ввод", Toast.LENGTH_SHORT).show();
            }
        });

        builder.setNegativeButton("Отмена", (dialog, which) -> dialog.cancel());

        builder.show();
    }

    private void addInvitationToFirestore(String eventId, String fullName) {
        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        if (currentUser != null && eventId != null) {
            String userId = currentUser.getUid();
            Map<String, Object> invitationData = new HashMap<>();
            invitationData.put("fullName", fullName);
            invitationData.put("isAccepted", false);
            // qrCodeString будет установлен после создания приглашения в базе данных

            FirebaseFirestore db = FirebaseFirestore.getInstance();
            db.collection("users").document(userId)
                    .collection("events").document(eventId)
                    .collection("invitations")
                    .add(invitationData)
                    .addOnSuccessListener(documentReference -> {
                        // Генерация QR-кода после успешного создания приглашения
                        String qrCodeString = Invitation.generateQRCodeString(userId, eventId, documentReference.getId());
                        documentReference
                                .update("qrCodeString", qrCodeString)
                                .addOnSuccessListener(aVoid -> {
                                    // Создание объекта приглашения с QR-кодом
                                    Invitation newInvitation = new Invitation(fullName, qrCodeString);
                                    newInvitation.setId(documentReference.getId());

                                    // Обновление UI в основном потоке
                                    runOnUiThread(() -> {
                                        invitations.add(newInvitation);
                                        adapter.notifyItemInserted(invitations.size() - 1);
                                    });
                                })
                                .addOnFailureListener(e -> Log.w("InvitationsActivity", "Error setting QR code for invitation", e));

                        Toast.makeText(InvitationsActivity.this, "Приглашение добавлено", Toast.LENGTH_SHORT).show();
                    })
                    .addOnFailureListener(e -> {
                        Toast.makeText(InvitationsActivity.this, "Ошибка при добавлении приглашения", Toast.LENGTH_SHORT).show();
                    });
        } else {
            Toast.makeText(this, "Вы должны авторизоваться", Toast.LENGTH_SHORT).show();
        }
    }

    private void loadInvitations(String eventId) {
        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        if (currentUser != null) {
            String userId = currentUser.getUid();
            db.collection("users").document(userId).collection("events").document(eventId)
                    .collection("invitations")
                    .orderBy("fullName") // Добавляем сортировку по полю fullName
                    .get()
                    .addOnSuccessListener(queryDocumentSnapshots -> {
                        invitations.clear(); // Очищаем список перед заполнением
                        for (QueryDocumentSnapshot documentSnapshot : queryDocumentSnapshots) {
                            Invitation invitation = documentSnapshot.toObject(Invitation.class);
                            if (invitation != null) {
                                invitation.setId(documentSnapshot.getId());
                                String qrCodeString = Invitation.generateQRCodeString(userId, eventId, documentSnapshot.getId());
                                DocumentReference invitationRef = db.collection("users").document(userId)
                                        .collection("events").document(eventId)
                                        .collection("invitations").document(documentSnapshot.getId());

                                invitationRef
                                        .update("qrCodeString", qrCodeString)
                                        .addOnSuccessListener(aVoid -> {
                                            invitation.setQrCodeString(qrCodeString);
                                            invitations.add(invitation);
                                            adapter.notifyItemInserted(invitations.size() - 1); // Обновляем адаптер для только что добавленного приглашения
                                        })
                                        .addOnFailureListener(e -> Log.w("InvitationsActivity", "Error setting QR code for invitation", e));
                            }
                        }
                    })
                    .addOnFailureListener(e -> Toast.makeText(InvitationsActivity.this, "Ошибка загрузки приглашений", Toast.LENGTH_SHORT).show());
        } else {
            Toast.makeText(this, "Вы должны авторизоваться", Toast.LENGTH_SHORT).show();
        }
    }



    private void showInvitationActionsDialog(Invitation invitation) {
        CharSequence[] actions = {"Показать QR-код", "Отправить QR-код", "Удалить"};

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setItems(actions, (dialog, which) -> {
            switch (which) {
                case 0: // Show QR Code
                    showQRCode(invitation);
                    break;
                case 1: // Send QR Code by Email
                    fetchEventDetailsAndSendEmail(invitation);
                    break;
                case 2: // Delete Invitation
                    deleteInvitation(invitation);
                    break;
            }
        });
        builder.show();
    }
    private void fetchEventDetailsAndSendEmail(Invitation invitation) {
        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        if (currentUser != null && eventId != null) {
            FirebaseFirestore db = FirebaseFirestore.getInstance();
            db.collection("users").document(currentUser.getUid()).collection("events").document(eventId)
                    .get()
                    .addOnSuccessListener(documentSnapshot -> {
                        if (documentSnapshot.exists()) {
                            String eventName = documentSnapshot.getString("name");
                            String eventDate = documentSnapshot.getString("date");
                            String eventTime = documentSnapshot.getString("time");
                            String eventPlace = documentSnapshot.getString("place");
                            sendQRCodeByEmail(invitation, eventName, eventDate, eventTime, eventPlace); // Теперь отправляем письмо с названием и датой мероприятия
                        } else {
                            Log.d("InvitationsActivity", "Event not found");
                        }
                    })
                    .addOnFailureListener(e -> Log.w("InvitationsActivity", "Error getting event details", e));
        }
    }

    @Override
    public void onInvitationClick(Invitation invitation) {
        showInvitationActionsDialog(invitation);
    }
    private void showQRCode(Invitation invitation) {
        // Декодирование QR-кода из строки
        byte[] decodedString = Base64.decode(invitation.getQrCodeString(), Base64.DEFAULT);
        Bitmap decodedByte = BitmapFactory.decodeByteArray(decodedString, 0, decodedString.length);

        // Создание ImageView для отображения QR-кода
        ImageView imageView = new ImageView(this);
        imageView.setImageBitmap(decodedByte);

        // Создание диалогового окна для отображения изображения на весь экран
        final Dialog dialog = new Dialog(this);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.setContentView(imageView);

        // Установка параметров Layout для ImageView, чтобы QR-код был на весь экран
        imageView.setLayoutParams(new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

        // Добавление обработчика клика по изображению для закрытия диалога
        imageView.setOnClickListener(v -> dialog.dismiss());

        // Показ диалога на весь экран
        dialog.show();

        // Установка параметров окна диалога для отображения на весь экран
        Window window = dialog.getWindow();
        if (window != null) {
            window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
            window.setBackgroundDrawable(new ColorDrawable(android.graphics.Color.TRANSPARENT));
        }
    }



    private void sendQRCodeByEmail(Invitation invitation, String eventName, String eventDate, String eventTime, String eventPlace) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);

        final EditText emailInput = new EditText(this);
        emailInput.setInputType(InputType.TYPE_TEXT_VARIATION_EMAIL_ADDRESS);
        emailInput.setHint("Введите email(если нужно)");
        builder.setView(emailInput);

        builder.setPositiveButton("Отправить", (dialog, which) -> {
            String emailAddress = emailInput.getText().toString().trim();
            createAndSendEmail(emailAddress, invitation, eventName, eventDate, eventTime, eventPlace);
        });

        builder.setNegativeButton("Отмена", (dialog, which) -> dialog.cancel());
        builder.show();
    }

    private void createAndSendEmail(String emailAddress, Invitation invitation, String eventName, String eventDate, String eventTime, String eventPlace) {
        ExecutorService emailExecutor = Executors.newSingleThreadExecutor();
        Handler handler = new Handler(Looper.getMainLooper());

        emailExecutor.execute(() -> {
            byte[] decodedString = Base64.decode(invitation.getQrCodeString(), Base64.DEFAULT);
            Bitmap qrCodeBitmap = BitmapFactory.decodeByteArray(decodedString, 0, decodedString.length);
            Bitmap combinedBitmap = createImageWithTextAndQrCode(eventName, eventDate, eventTime, eventPlace, qrCodeBitmap);

            handler.post(() -> sendEmailWithBitmap(emailAddress, combinedBitmap, invitation.getFullName(), eventName, eventDate, eventTime, eventPlace));
        });
    }

    private Bitmap createImageWithTextAndQrCode(String eventName, String eventDate, String eventTime, String eventPlace, Bitmap qrCodeBitmap) {
        // Вычисляем высоту текстовой области исходя из размера текста и интервалов
        Paint textPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        textPaint.setColor(Color.BLACK);
        textPaint.setTextSize(40);
        textPaint.setTextAlign(Paint.Align.LEFT); // Изменено выравнивание на LEFT

        // Высота для четырех строк текста и некоторого паддинга сверху и снизу
        int textHeight = (int) (textPaint.descent() - textPaint.ascent()) * 4+6;

        // Создание нового изображения, достаточно большого, чтобы вместить текст и QR-код
        Bitmap combinedBitmap = Bitmap.createBitmap(qrCodeBitmap.getWidth(), qrCodeBitmap.getHeight() + textHeight, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(combinedBitmap);

        // Заливаем область для текста белым цветом
        Paint backgroundPaint = new Paint();
        backgroundPaint.setColor(Color.WHITE);
        canvas.drawRect(0, 0, combinedBitmap.getWidth(), textHeight, backgroundPaint);

        // Рисуем текст на холсте
        float x = 20; // Начальная позиция X с небольшим отступом слева
        float y = 40; // Начальная позиция Y с небольшим отступом сверху
        canvas.drawText("Мероприятие: " + eventName, x, y, textPaint);
        y += textPaint.descent() - textPaint.ascent(); // Отступ между строками
        canvas.drawText("Дата: " + eventDate, x, y, textPaint);
        y += textPaint.descent() - textPaint.ascent();
        canvas.drawText("Время: " + eventTime, x, y, textPaint);
        y += textPaint.descent() - textPaint.ascent();
        canvas.drawText("Место: " + eventPlace, x, y, textPaint);

        // Рисуем QR-код ниже текста
        canvas.drawBitmap(qrCodeBitmap, 0, textHeight, null);

        return combinedBitmap;
    }


    private void sendEmailWithBitmap(String emailAddress, Bitmap bitmap, String fullName, String eventName, String eventDate, String eventTime, String eventPlace ) {
        // Создание URI из битмапа
        String path = MediaStore.Images.Media.insertImage(getContentResolver(), bitmap, "QR Code", null);
        Uri bmpUri = Uri.parse(path);

        // Создание намерения для отправки письма
        Intent emailIntent = new Intent(Intent.ACTION_SEND);
        emailIntent.setType("image/png");
        emailIntent.putExtra(Intent.EXTRA_EMAIL, new String[]{emailAddress});
        emailIntent.putExtra(Intent.EXTRA_SUBJECT, "Приглашение на " + eventName);
        emailIntent.putExtra(Intent.EXTRA_TEXT, "Здравствуйте, " + fullName + ", вы были приглашены на мероприятие " + eventName + ".\nВремя: " + eventDate+ " " + eventTime+".\nМесто проведения: "+eventPlace+".\nВаш QR-code прикреплен к сообщению:");
        emailIntent.putExtra(Intent.EXTRA_STREAM, bmpUri);
        startActivity(Intent.createChooser(emailIntent, "Send email..."));
        // Удаление изображения после задержки в 20 секунд
        new Handler(Looper.getMainLooper()).postDelayed(() -> getContentResolver().delete(bmpUri, null, null), 20000);
    }



    // Пример метода для удаления приглашения
    private void deleteInvitation(Invitation invitation) {
        FirebaseUser currentUser = FirebaseAuth.getInstance().getCurrentUser();
        if (currentUser != null && invitation.getId() != null && eventId != null) {
            String userId = currentUser.getUid();

            db.collection("users").document(userId)
                    .collection("events").document(eventId)
                    .collection("invitations").document(invitation.getId())
                    .delete()
                    .addOnSuccessListener(aVoid -> {
                        // Находим позицию приглашения в списке
                        int position = invitations.indexOf(invitation);
                        if (position > -1) {
                            // Удаление приглашения из списка и оповещение адаптера
                            invitations.remove(position);
                            adapter.notifyItemRemoved(position);
                        }
                        Toast.makeText(InvitationsActivity.this, "Приглашение удалено", Toast.LENGTH_SHORT).show();
                    })
                    .addOnFailureListener(e -> {
                        Toast.makeText(this, "Ошибка при удалении", Toast.LENGTH_SHORT).show();
                    });
        } else {
            Toast.makeText(this, "Потеряна информация", Toast.LENGTH_SHORT).show();
        }
    }


}
